package com.ycwl.basic.device.repository;

import com.alibaba.fastjson.JSONObject;
import com.ycwl.basic.mapper.DeviceMapper;
import com.ycwl.basic.model.pc.device.entity.DeviceConfigEntity;
import com.ycwl.basic.model.pc.device.entity.DeviceEntity;
import com.ycwl.basic.utils.SnowFlakeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

@Component
public class DeviceRepository {
    @Autowired
    private DeviceMapper deviceMapper;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    public static final String DEVICE_CACHE_KEY = "device:%s";
    public static final String DEVICE_CONFIG_CACHE_KEY = "device:%s:config";

    public DeviceEntity getDevice(Long deviceId) {
        if (redisTemplate.hasKey(String.format(DEVICE_CACHE_KEY, deviceId))) {
            return JSONObject.parseObject(redisTemplate.opsForValue().get(String.format(DEVICE_CACHE_KEY, deviceId)), DeviceEntity.class);
        }
        DeviceEntity device = deviceMapper.getByDeviceId(deviceId);
        if (null != device) {
            redisTemplate.opsForValue().set(String.format(DEVICE_CACHE_KEY, deviceId), JSONObject.toJSONString(device));
        }
        return device;
    }

    public DeviceEntity getDeviceByDeviceNo(String deviceNo) {
        if (redisTemplate.hasKey(String.format(DEVICE_CACHE_KEY, deviceNo))) {
            return JSONObject.parseObject(redisTemplate.opsForValue().get(String.format(DEVICE_CACHE_KEY, deviceNo)), DeviceEntity.class);
        }
        DeviceEntity device = deviceMapper.getByDeviceNo(deviceNo);
        if (null != device) {
            redisTemplate.opsForValue().set(String.format(DEVICE_CACHE_KEY, deviceNo), JSONObject.toJSONString(device));
        }
        return device;
    }

    public DeviceConfigEntity getDeviceConfig(Long deviceId) {
        if (redisTemplate.hasKey(String.format(DEVICE_CONFIG_CACHE_KEY, deviceId))) {
            return JSONObject.parseObject(redisTemplate.opsForValue().get(String.format(DEVICE_CONFIG_CACHE_KEY, deviceId)), DeviceConfigEntity.class);
        }
        DeviceConfigEntity deviceConfig = deviceMapper.getConfigByDeviceId(deviceId);
        if (null == deviceConfig) {
            deviceConfig = new DeviceConfigEntity();
            deviceConfig.setId(SnowFlakeUtil.getLongId());
            deviceConfig.setDeviceId(deviceId);
            deviceMapper.addConfig(deviceConfig);
        }
        redisTemplate.opsForValue().set(String.format(DEVICE_CONFIG_CACHE_KEY, deviceId), JSONObject.toJSONString(deviceConfig));
        return deviceConfig;
    }

    public boolean clearDeviceCache(Long deviceId) {
        if (redisTemplate.hasKey(String.format(DEVICE_CACHE_KEY, deviceId))) {
            DeviceEntity device = getDevice(deviceId);
            redisTemplate.delete(String.format(DEVICE_CACHE_KEY, device.getNo()));
            redisTemplate.delete(String.format(DEVICE_CONFIG_CACHE_KEY, device.getNo()));
        }
        redisTemplate.delete(String.format(DEVICE_CACHE_KEY, deviceId));
        redisTemplate.delete(String.format(DEVICE_CONFIG_CACHE_KEY, deviceId));
        return true;
    }
}
