package com.ycwl.basic.service.impl.mobile;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ycwl.basic.constant.BaseContextHandler;
import com.ycwl.basic.mapper.*;
import com.ycwl.basic.model.mobile.goods.*;
import com.ycwl.basic.model.pc.face.resp.FaceRespVO;
import com.ycwl.basic.model.pc.scenic.resp.ScenicRespVO;
import com.ycwl.basic.model.pc.source.req.SourceReqQuery;
import com.ycwl.basic.model.pc.source.resp.SourceRespVO;
import com.ycwl.basic.model.pc.task.entity.TaskEntity;
import com.ycwl.basic.model.pc.task.resp.TaskRespVO;
import com.ycwl.basic.model.pc.video.entity.MemberVideoEntity;
import com.ycwl.basic.model.pc.video.req.VideoReqQuery;
import com.ycwl.basic.model.pc.video.resp.VideoRespVO;
import com.ycwl.basic.repository.VideoTaskRepository;
import com.ycwl.basic.service.mobile.GoodsService;
import com.ycwl.basic.repository.TemplateRepository;
import com.ycwl.basic.utils.ApiResponse;
import com.ycwl.basic.utils.DateUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @Author：longbinbin
 * @Date：2024/12/5 15:04
 */@Service
public class GoodsServiceImpl implements GoodsService {
     @Autowired
     private VideoMapper videoMapper;
     @Autowired
     private SourceMapper sourceMapper;
     @Autowired
     private ScenicMapper scenicMapper;
     @Autowired
     private TaskMapper taskMapper;
     @Autowired
     private DeviceMapper deviceMapper;
    @Autowired
    private FaceMapper faceMapper;
     @Autowired
     private TemplateRepository templateRepository;
    @Autowired
    private VideoTaskRepository videoTaskRepository;

    public ApiResponse<List<GoodsPageVO>> goodsList(GoodsReqQuery query) {
         //查询原素材
         List<GoodsPageVO> goodsList = new ArrayList<>();

         VideoReqQuery videoReqQuery = new VideoReqQuery();
         videoReqQuery.setScenicId(query.getScenicId());
         videoReqQuery.setIsBuy(query.getIsBuy());
         videoReqQuery.setMemberId(Long.valueOf(BaseContextHandler.getUserId()));
         //查询成片vlog
         List<VideoRespVO> videoList = videoMapper.queryByRelation(videoReqQuery);
         videoList.forEach(videoRespVO -> {
             GoodsPageVO goodsPageVO = new GoodsPageVO();
             goodsPageVO.setGoodsName(videoRespVO.getTemplateName());
             goodsPageVO.setScenicId(videoRespVO.getScenicId());
             goodsPageVO.setScenicName(videoRespVO.getScenicName());
             goodsPageVO.setGoodsType(0);
             goodsPageVO.setFaceId(videoRespVO.getFaceId());
             goodsPageVO.setGoodsId(videoRespVO.getId());
             goodsPageVO.setTemplateName(videoRespVO.getTemplateName());
             goodsPageVO.setTemplateCoverUrl(videoRespVO.getTemplateCoverUrl());
             goodsList.add(goodsPageVO);
         });


         SourceReqQuery sourceReqQuery = new SourceReqQuery();
         sourceReqQuery.setScenicId(query.getScenicId());
         sourceReqQuery.setIsBuy(query.getIsBuy());
         sourceReqQuery.setMemberId(Long.valueOf(BaseContextHandler.getUserId()));
         //查询源素材
         List<SourceRespVO> sourceList = sourceMapper.queryByRelation(sourceReqQuery);
        List<GoodsPageVO> sourceGoods = sourceList.stream().collect(Collectors.groupingBy(SourceRespVO::getFaceId)).entrySet().stream().flatMap((faceEntry) -> {
            Long faceId = faceEntry.getKey();
            List<SourceRespVO> goods = faceEntry.getValue();
            return goods.stream().collect(Collectors.groupingBy(SourceRespVO::getType)).keySet().stream().map(type -> {
                GoodsPageVO goodsPageVO = new GoodsPageVO();
                goodsPageVO.setFaceId(faceId);
                if (type == 1) {
                    goodsPageVO.setGoodsName("原片集");
                    goodsPageVO.setGoodsType(1);
                } else {
                    goodsPageVO.setGoodsName("照片集");
                    goodsPageVO.setGoodsType(2);
                }
                goodsPageVO.setScenicId(query.getScenicId());
                return goodsPageVO;
            });
        }).collect(Collectors.toList());
        if (!sourceGoods.isEmpty()) {
            if (goodsList.size() > 2) {
                goodsList.addAll(2, sourceGoods);
            } else {
                goodsList.addAll(sourceGoods);
            }
        }
        return ApiResponse.success(goodsList);
     }

    @Override
    public ApiResponse<List<GoodsDetailVO>> sourceGoodsList(GoodsReqQuery query) {
        Integer sourceType = query.getSourceType();
        SourceReqQuery sourceReqQuery = new SourceReqQuery();
        sourceReqQuery.setScenicId(query.getScenicId());
        sourceReqQuery.setIsBuy(query.getIsBuy());
        sourceReqQuery.setMemberId(query.getMemberId());
        sourceReqQuery.setType(sourceType);
        sourceReqQuery.setFaceId(query.getFaceId());
        List<SourceRespVO> list = sourceMapper.listUser(sourceReqQuery);
        List<GoodsDetailVO> goodsDetailVOList = new ArrayList<>();

        String goodsNamePrefix = "";
        if(sourceType==1){
            goodsNamePrefix="视频";
        }else if (sourceType==2){
            goodsNamePrefix="图片";
        }else {
            goodsNamePrefix="其他类型";
        }
        //图片编号
        int i=1;
        for (SourceRespVO sourceRespVO : list) {
            GoodsDetailVO goodsDetailVO = new GoodsDetailVO();
            goodsDetailVO.setGoodsId(sourceRespVO.getId());
            String shootingTime = DateUtils.format(sourceRespVO.getCreateTime(), "yyyy.MM.dd HH:mm:ss");
            if(i<10){
                goodsDetailVO.setGoodsName(goodsNamePrefix +"0"+i+" "+shootingTime);
            }else {
                goodsDetailVO.setGoodsName(goodsNamePrefix +i+" "+shootingTime);
            }
            goodsDetailVO.setScenicId(sourceRespVO.getScenicId());
            goodsDetailVO.setScenicName(sourceRespVO.getScenicName());
            goodsDetailVO.setLongitude(sourceRespVO.getLongitude());
            goodsDetailVO.setLatitude(sourceRespVO.getLatitude());
            goodsDetailVO.setGoodsType(sourceType);
            goodsDetailVO.setSourceType(sourceType);
            goodsDetailVO.setGoodsId(sourceRespVO.getId());
            goodsDetailVO.setVideoUrl(sourceRespVO.getVideoUrl());
            goodsDetailVO.setUrl(sourceRespVO.getUrl());
            goodsDetailVO.setCreateTime(sourceRespVO.getCreateTime());

            goodsDetailVOList.add(goodsDetailVO);
            i++;
        }

        return ApiResponse.success(goodsDetailVOList);
    }

    @Override
    public ApiResponse<BigDecimal> queryPrice(GoodsPriceQueryReq queryPriceData) {
        Integer goodsType = queryPriceData.getGoodsType();

        if(goodsType==0){//成片vlog
            //成片的价格就是成片所用template的价格
            Long videoId = queryPriceData.getGoodsId();
            VideoRespVO videoRespVO = videoMapper.getById(videoId);
            if(videoRespVO==null){
                return ApiResponse.fail("该vlog不存在或已失效");
            }
            BigDecimal templatePrice = videoRespVO.getTemplatePrice();
            if(templatePrice==null){
                return ApiResponse.fail("该vlog使用的模板价格或状态异常，请联系管理员");
            }
            return ApiResponse.success(templatePrice);

        }else if(goodsType==1 || goodsType == 2){//原素材
            //原素材的价格就是原素材对应景区定的价格
            Long scenicId = queryPriceData.getScenicId();
            ScenicRespVO scenicRespVO = scenicMapper.getById(scenicId);
            if(scenicRespVO==null){
                return ApiResponse.fail("该景区不存在或状态异常，请联系管理员");
            }
            BigDecimal price = scenicRespVO.getPrice();
            if(price==null){
                return ApiResponse.fail("该景区的原片价格未设定或状态异常，请联系管理员");
            }
            return ApiResponse.success(price);
        }
        return ApiResponse.fail("不合法的商品，请联系管理员");
    }

    @Override
    public ApiResponse<VideoGoodsDetailVO> videoGoodsDetail(Long userId, Long videoId) {
        MemberVideoEntity entity = videoMapper.queryUserVideo(userId, videoId);
        if (entity == null) {
            return ApiResponse.fail("该视频不属于您");
        }
        VideoGoodsDetailVO goodsDetailVO = new VideoGoodsDetailVO();
        VideoRespVO videoRespVO = videoMapper.getById(videoId);
        if(videoRespVO==null){
            return ApiResponse.fail("该vlog不存在或已失效");
        }
        goodsDetailVO.setGoodsName(videoRespVO.getTemplateName());
        goodsDetailVO.setScenicId(videoRespVO.getScenicId());
        goodsDetailVO.setScenicName(videoRespVO.getScenicName());
        goodsDetailVO.setGoodsType(0);
        goodsDetailVO.setGoodsId(videoRespVO.getId());
        goodsDetailVO.setVideoUrl(videoRespVO.getVideoUrl());
        goodsDetailVO.setTemplateCoverUrl(videoRespVO.getTemplateCoverUrl());
        goodsDetailVO.setCreateTime(videoRespVO.getCreateTime());
        if (entity.getIsBuy() == 1) {
            goodsDetailVO.setIsBuy(1);
        } else {
            goodsDetailVO.setIsBuy(0);
            BigDecimal templatePrice = videoRespVO.getTemplatePrice();
            BigDecimal slashPrice = videoRespVO.getSlashPrice();
            // 使用DecimalFormat格式化输出
            DecimalFormat df = new DecimalFormat("0.00");
            goodsDetailVO.setPrice(templatePrice==null?"":df.format(templatePrice.setScale(2, RoundingMode.HALF_UP)));
            goodsDetailVO.setSlashPrice(slashPrice==null?null:df.format(slashPrice.setScale(2, RoundingMode.HALF_UP)));
        }
        TaskRespVO taskRespVO = taskMapper.getById(videoRespVO.getTaskId());
        JSONObject paramJson = JSON.parseObject(taskRespVO.getTaskParams());
        long deviceCount;
        if (paramJson == null) {
            deviceCount = 1;
        } else {
            deviceCount = paramJson.keySet().stream().filter(StringUtils::isNumeric).count();
        }
        goodsDetailVO.setLensNum((int) deviceCount);
        goodsDetailVO.setFaceId(entity.getFaceId());
        return ApiResponse.success(goodsDetailVO);
    }

    /**
     * 查询用户当前景区的视频合成任务状态
     *
     * @param faceId 景区id
     * @return 0没有任务 1 合成中 2 合成成功
     */
    @Override
    public ApiResponse<VideoTaskStatusVO> getTaskStatusByFaceId(Long userId, Long faceId) {
        List<MemberVideoEntity> taskList = videoMapper.listRelationByFace(userId, faceId);
        VideoTaskStatusVO response = new VideoTaskStatusVO();
        response.setFaceId(faceId);
        if (taskList.isEmpty()) {
            response.setStatus(0);
            return ApiResponse.success(response);
        }
        response.setScenicId(taskList.get(0).getScenicId());
        response.setMaxCount(templateRepository.getTemplateListByScenicId(response.getScenicId()).size());
        List<MemberVideoEntity> notFinishedTasks = taskList.stream().filter(task -> {
            TaskEntity taskById = videoTaskRepository.getTaskById(task.getTaskId());
            if (taskById == null) {
                return true;
            }
            return taskById.getStatus() != 1;
        }).collect(Collectors.toList());
        if (!notFinishedTasks.isEmpty()) {
            response.setCount(taskList.size() - notFinishedTasks.size());
            response.setTemplateId(notFinishedTasks.get(0).getTemplateId());
            response.setTaskId(notFinishedTasks.get(0).getTaskId());
            response.setStatus(2);
            return ApiResponse.success(response);
        }
        MemberVideoEntity lastVideo = taskList.get(taskList.size() - 1);
        response.setTaskId(lastVideo.getTaskId());
        response.setTemplateId(lastVideo.getTemplateId());
        response.setVideoId(lastVideo.getVideoId());
        response.setCount(taskList.size());
        response.setStatus(1);
        return ApiResponse.success(response);
    }

    @Override
    public ApiResponse<VideoTaskStatusVO> getAllTaskStatus(Long userId) {
        FaceRespVO lastFaceByUserId = faceMapper.findLastFaceByUserId(String.valueOf(userId));
        return getTaskStatusByFaceId(userId, lastFaceByUserId.getId());
    }

    @Override
    public ApiResponse<VideoTaskStatusVO> getTaskStatusByTemplateId(Long userId, Long faceId, Long templateId) {
        List<MemberVideoEntity> taskList = videoMapper.listRelationByFaceAndTemplate(userId, faceId, templateId);
        VideoTaskStatusVO response = new VideoTaskStatusVO();
        response.setFaceId(faceId);
        if (taskList.isEmpty()) {
            response.setStatus(0);
            return ApiResponse.success(response);
        }
        response.setScenicId(taskList.get(0).getScenicId());
        response.setMaxCount(templateRepository.getTemplateListByScenicId(response.getScenicId()).size());
        List<MemberVideoEntity> notFinishedTasks = taskList.stream().filter(task -> {
            TaskEntity taskById = videoTaskRepository.getTaskById(task.getTaskId());
            if (taskById == null) {
                return true;
            }
            return taskById.getStatus() != 1;
        }).collect(Collectors.toList());
        if (!notFinishedTasks.isEmpty()) {
            response.setCount(taskList.size() - notFinishedTasks.size());
            response.setTemplateId(notFinishedTasks.get(0).getTemplateId());
            response.setTaskId(notFinishedTasks.get(0).getTaskId());
            response.setStatus(2);
            return ApiResponse.success(response);
        }
        MemberVideoEntity lastVideo = taskList.get(taskList.size() - 1);
        response.setTaskId(lastVideo.getTaskId());
        response.setTemplateId(lastVideo.getTemplateId());
        response.setVideoId(lastVideo.getVideoId());
        response.setCount(taskList.size());
        response.setStatus(1);
        return ApiResponse.success(response);
    }

    @Override
    public ApiResponse<VideoTaskStatusVO> getTaskStatusByScenicId(Long userId, Long scenicId) {
        FaceRespVO faceVO = faceMapper.getByMemberId(userId, scenicId);
        VideoTaskStatusVO response = new VideoTaskStatusVO();
        response.setScenicId(scenicId);
        if (faceVO == null) {
            // 从来没露脸
            response.setStatus(-2);
            return ApiResponse.success(response);
        }
        return getTaskStatusByFaceId(userId, faceVO.getId());
    }

    @Override
    public ApiResponse<GoodsDetailVO> sourceGoodsInfo(Long userId, Long sourceId) {
        SourceRespVO sourceRespVO = sourceMapper.listUserOne(userId, sourceId);
//        if (sourceRespVO == null) {
//            return ApiResponse.fail("该视频不属于你");
//        }
        GoodsDetailVO goodsDetailVO = new GoodsDetailVO();
        goodsDetailVO.setGoodsName("原片");
        goodsDetailVO.setScenicId(sourceRespVO.getScenicId());
        goodsDetailVO.setScenicName(sourceRespVO.getScenicName());
        goodsDetailVO.setGoodsType(0);
        goodsDetailVO.setGoodsId(sourceRespVO.getId());
        goodsDetailVO.setVideoUrl(sourceRespVO.getVideoUrl());
        goodsDetailVO.setTemplateCoverUrl(sourceRespVO.getUrl());
        goodsDetailVO.setCreateTime(sourceRespVO.getCreateTime());
        return ApiResponse.success(goodsDetailVO);
    }

}
